#ifndef _MX1_IDE_H_
#define _MX1_IDE_H_

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(vers,rel,seq) ( ((vers)<<16) | ((rel)<<8) | (seq) )
#endif


/* only allow 2.4.x kernels */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) /* version < 2.4.x */
#error "Kernel too old: not supported by this file, please use 2.4.x kernel."
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0) /* version > 2.4.x */
#error "Kernel too recent: not supported by this file, please use 2.4.x kernel."
#endif


/* specific includes */
#include "common.h"
#ifdef MW_KERNEL
/* Metrowerks missing definitions */
/* implementation in progress
#include <asm/arch/platform.h>
#define PTA_BASE_ADDR           GPIO_PTA_BASE
#define PTA_DDIR                PTA_BASE_ADDR
#define PTA_GIUS                (PTA_BASE_ADDR+0x20)
#define PTA_OCR1                (PTA_BASE_ADDR+0x04)
#define PTA_ICONFA2             (PTA_BASE_ADDR+0x10)
#define PTA_PUEN                (PTA_BASE_ADDR+0x40)
#define PTA_DR                  (PTA_BASE_ADDR+0x1C)
#define PTA_ICR2                (PTA_BASE_ADDR+0x2C)
#define PTA_ISR                 (PTA_BASE_ADDR+0x34)
#define PTA_IMR                 (PTA_BASE_ADDR+0x30)
#define PTA_GIUS                (PTA_BASE_ADDR+0x20)
#define PTB_BASE_ADDR           GPIO_PTB_BASE                
#define PTB_DDIR                PTB_BASE_ADDR
#define PTB_GIUS                (PTB_BASE_ADDR+0x20)
#define PTB_OCR2                (PTB_BASE_ADDR+0x08)
#define PTB_PUEN                (PTB_BASE_ADDR+0x40)
#define PTB_DR                  (PTB_BASE_ADDR+0x1C)
#define EIM_BASE_ADDR           EIM_BASE
#define EIM_CS5H                (EIM_BASE_ADDR+0x28)
#define CRM_BASE_ADDR           PLL_CSCR                
#define CRM_CSCR                CRM_BASE_ADDR
*/

/* MX1/L register address in virtual kernel space */
/* implementation in progress
#define PTA_GIUS_     IO_ADDRESS(PTA_GIUS)
#define PTA_OCR1_     IO_ADDRESS(PTA_OCR1)
#define PTA_ICONFA2_  IO_ADDRESS(PTA_ICONFA2)
#define PTA_DDIR_     IO_ADDRESS(PTA_DDIR)
#define PTA_PUEN_     IO_ADDRESS(PTA_PUEN)
#define PTA_DR_       IO_ADDRESS(PTA_DR)
#define PTA_ICR2_     IO_ADDRESS(PTA_ICR2)
#define PTA_ISR_      IO_ADDRESS(PTA_ISR)
#define PTA_SSR_      IO_ADDRESS(PTA_SSR)
#define PTA_IMR_      IO_ADDRESS(PTA_IMR)
#define PTB_GIUS_     IO_ADDRESS(PTB_GIUS)
#define PTB_OCR2_     IO_ADDRESS(PTB_OCR2)
#define PTB_DDIR_     IO_ADDRESS(PTB_DDIR)
#define PTB_PUEN_     IO_ADDRESS(PTB_PUEN)
#define PTB_DR_       IO_ADDRESS(PTB_DR)
#define EIM_CS5H_     IO_ADDRESS(EIM_CS5H)
#define EIM_CS5L_     IO_ADDRESS(EIM_CS5L)
#define CRM_CSCR_     IO_ADDRESS(CRM_CSCR)

#define      IDE_MEM_BASE     IO_ADDRESS(0xf6000000)   */         /* ide drive register memory offset in virtual kernel space */

#else
#include "mx1_def.h"
/* memory offsets and variables */
#define OFS_HVP 0xf0000000             /* ADS hardware base offset */ 

#define      IDE_MEM_BASE      0xf6000000            /* ide drive register memory offset in virtual kernel space */

/* MX1/L register address in virtual kernel space */
#define PTA_GIUS_    (PTA_GIUS    +    OFS_HVP)
#define PTA_OCR1_    (PTA_OCR1    +    OFS_HVP)
#define PTA_ICONFA2_ (PTA_ICONFA2 +    OFS_HVP)
#define PTA_DDIR_    (PTA_DDIR    +    OFS_HVP)
#define PTA_PUEN_    (PTA_PUEN    +    OFS_HVP)
#define PTA_DR_      (PTA_DR      +    OFS_HVP)
#define PTA_ICR2_    (PTA_ICR2    +    OFS_HVP)
#define PTA_ISR_     (PTA_ISR     +    OFS_HVP)
#define PTA_SSR_     (PTA_SSR     +    OFS_HVP)
#define PTA_IMR_     (PTA_IMR     +    OFS_HVP)
#define PTB_GIUS_    (PTB_GIUS    +    OFS_HVP)
#define PTB_OCR2_    (PTB_OCR2    +    OFS_HVP)
#define PTB_DDIR_    (PTB_DDIR    +    OFS_HVP)
#define PTB_PUEN_    (PTB_PUEN    +    OFS_HVP)
#define PTB_DR_      (PTB_DR      +    OFS_HVP)
#define EIM_CS5H_    (EIM_CS5H    +    OFS_HVP)
#define EIM_CS5L_    (EIM_CS5L    +    OFS_HVP)
#define CRM_CSCR_    (CRM_CSCR    +    OFS_HVP)

#endif

/* device properties */
#define MX1_IDE_MAJOR         0        /* dynamic major by default */
#define MX1_IDE_BLK_SIZE      0        /* 0 kB by default, autodetect must initialize hard drive size */
#define MX1_IDE_BLKSIZE_SIZE  1024     /* logical sector size, 1024 B */
#define MX1_IDE_HARDSECT_SIZE 512      /* hard sector size, 512 B */
#define MX1_IDE_READAHEAD     8        /* read ahead 8 sectors */
#define MX1_IDE_MAXS          2        /* max sectors of individual request */ /// 1
#define MX1_IDE_IRQ           11       /* IDE interrupt number */ 

/* IDE drive identity */
typedef struct ide_dev_id {
  unsigned short ide_ident[256];       /* ide drive ID */
  char ide_name[41];                   /* ide drive name */
  char ide_serial[21];                 /* ide drive serial */
  char ide_firmware[9];                /* ide drive firmware */
  unsigned int ide_sec_count;          /* ide drive sector number in CHS mode */
  unsigned int ide_sec_lba_count;      /* ide drive sector number in LBA mode */
  unsigned short ide_log_cyl;          /* ide drive logical cylinder number */
  unsigned short ide_log_head;         /* ide drive logical head number */
  unsigned short ide_sec_trk;          /* ide drive sectors per track number */
} ide_dev_id;

/* IDE command */
typedef struct ide_cmd {
  int command;                         /* command code */
  char *buffer;                        /* data buffer */
  unsigned int block;                  /* block address */
  unsigned int nsect;                  /* transfered sector number */
} ide_cmd;

/* IDE drive internal representation */
typedef struct ide_dev {
  int usage;                           /* device usage count */
  unsigned char busy;                  /* busy flag */
  unsigned char cmd_busy;              /* command busy flag */
  unsigned char error;                 /* error flag */
  unsigned char retry;                 /* retry counter */
  unsigned char wait_intq;             /* interrupt request flag */
  ide_cmd cmd;                         /* current command */
  spinlock_t lock_dev;                 /* lock variable */
  struct timer_list timer;             /* device timer */
} ide_dev;


/* IDE base registers */ 
#define      IDE_MEM_DIM       10                    /* size of allocated memory, allow to access ide device registers */
#define      IDE_DATA_REG      (IDE_MEM_BASE  +  0x10) /* ide data register */
#define      IDE_ERR_REG       (IDE_MEM_BASE  +  0x12) /* ide error register */
#define      IDE_FEAT_REG      (IDE_MEM_BASE  +  0x12) /* ide feature register */
#define      IDE_SEC_COUNT_REG (IDE_MEM_BASE  +  0x14) /* ide sector count register */
#define      IDE_SEC_NB_REG    (IDE_MEM_BASE  +  0x16) /* ide sector number register */
#define      IDE_CYL_LOW_REG   (IDE_MEM_BASE  +  0x18) /* ide low cylinder register */
#define      IDE_CYL_HIGH_REG  (IDE_MEM_BASE  +  0x1A) /* ide high cylinder register */
#define      IDE_HEAD_REG      (IDE_MEM_BASE  +  0x1C) /* ide head register */
#define      IDE_STAT_REG      (IDE_MEM_BASE  +  0x1E) /* ide status register */
#define      IDE_CMD_REG       (IDE_MEM_BASE  +  0x1E) /* ide command register */
#define      IDE_ALT_STAT_REG  (IDE_MEM_BASE  +  0x2C) /* ide alternative status register */
#define      IDE_ADD_REG       (IDE_MEM_BASE  +  0x2E) /* ide device address register */

/* IDE commands */
#define      WAIT_BSY_DRDY      mx1_IDE_wait_signal(0x3f, 0x40)
#define      WAIT_BSY           mx1_IDE_wait_signal(0x7f, 0x0)
#define      WAIT_BSY_DRQ       mx1_IDE_wait_signal(0x77, 0x8)
#define      IDE_READ_MULT      0xc4
#define      IDE_WRITE_MULT     0xc5
#define      IDE_READ           0x20
#define      IDE_WRITE          0x30
#define      IDE_DEV_DIAG       0x90
#define      IDE_SET_MULT       0xc6 
#define      IDE_IDENT_DEV      0xec
#define      IDE_SET_FEAT       0xef
#define      IDE_SET_HEAD_REG   outb_p(0xe0,          (volatile P_U8) IDE_HEAD_REG) 
#define      IDE_READ_CMD       outb_p(IDE_READ,      (volatile P_U8) IDE_CMD_REG)
#define      IDE_WRITE_CMD      outb_p(IDE_WRITE,     (volatile P_U8) IDE_CMD_REG)
#define      IDE_READ_MULT_CMD  outb_p(IDE_READ_MULT, (volatile P_U8) IDE_CMD_REG)
#define      IDE_WRITE_MULT_CMD outb_p(IDE_WRITE_MULT,(volatile P_U8) IDE_CMD_REG)
#define      IDE_DEV_DIAG_CMD   outb_p(IDE_DEV_DIAG,  (volatile P_U8) IDE_CMD_REG)
#define      IDE_SET_MULT_CMD   outb_p(IDE_SET_MULT,  (volatile P_U8) IDE_CMD_REG)
#define      IDE_IDENT_DEV_CMD  outb_p(IDE_IDENT_DEV, (volatile P_U8) IDE_CMD_REG)
#define      IDE_SET_FEAT_CMD   outb_p(IDE_SET_FEAT,  (volatile P_U8) IDE_CMD_REG)

/* OTHER */
#define      DEV_BUSY           mx1_ide_dev->busy
#define      SET_DEV_BUSY       mx1_ide_dev->busy = 1
#define      CLEAR_DEV_BUSY     mx1_ide_dev->busy = 0
#define      CMD_BUSY           mx1_ide_dev->cmd_busy
#define      SET_CMD_BUSY       mx1_ide_dev->cmd_busy = 1
#define      CLEAR_CMD_BUSY     mx1_ide_dev->cmd_busy = 0
#define      WAIT_INT           mx1_ide_dev->wait_intq
#define      SET_WAIT_INT       mx1_ide_dev->wait_intq = 1
#define      CLEAR_WAIT_INT     mx1_ide_dev->wait_intq = 0
#define      DEV_ERROR          mx1_ide_dev->error
#define      SET_DEV_ERROR      mx1_ide_dev->error = 1
#define      CLEAR_DEV_ERROR    mx1_ide_dev->error = 0
#define      LED2_ON            *(volatile P_U32) PTA_DR_  |= 0x4         /* led2 turn on */
#define      LED2_OFF           *(volatile P_U32) PTA_DR_  &= 0xfffffffb  /* led2 turn off */

#endif /*_MX1_IDE_H_ */ 
